package htmlvalidator;

import java.io.*;
import java.net.*;
import java.util.logging.Level;
import java.util.logging.Logger;

public class NuValidator extends Validator implements XValidator
{
    public NuValidator (String url) {
        this.validatorUrl = url;
        this.validatorCharset = "UTF-8";
        
        this.validatorPathFile = "%s/?out=text";
        this.validatorPathUri = "%s/?out=text&doc=%s";
    }
    
    // reads a file and send in using HTTP POST
    @Override
    public String checkByFile (String filepath) {
        HttpURLConnection connection = null;
        String results = ""; // response from the validation service
        
        try {
            String completeUri = String.format(this.validatorPathFile, 
                    this.validatorUrl);
            URL url = new URL(completeUri);
            connection = (HttpURLConnection) url.openConnection(); 
            StringBuilder sbResults = new StringBuilder();
            
            String line;
            StringBuilder sbFile = new StringBuilder();
            
            // prepare the connection 
            connection.setRequestMethod("POST");
            connection.setReadTimeout(15 * 1000);
            connection.setDoOutput(true);
            
            /* ===== nu validator specifics ===== */
            
            // Validator.nu wants the content of the file as the 
            // content-body of the HTTP POST request. 
            // We therefore read the file to memory and write it via an
            // output stream.
            
            FileInputStream fis = new FileInputStream(filepath);
            BufferedInputStream bis = new BufferedInputStream(fis);
            DataInputStream dis = new DataInputStream(bis);

            // for some reason, the validator chokes this:
            // "Error: Forbidden code point U+0019."
            // when we use BufferedReader.readLine. As long as we have this
            // situation, we are forced to use deprecated code.
            while (dis.available() != 0) {
                // TODO fix!
                line = dis.readLine();
                sbFile.append(line).append("\n");
            }
            
            fis.close();
            bis.close();
            dis.close();
            
            // prevent validation service from HTTP reponse code 415.
            connection.setRequestProperty("Content-Type", 
                    "text/html; charset=" + this.validatorCharset);
            
            //ok, now we got the whole file in memory.
            String body = sbFile.toString();
            
            /* ===== /nu validator specifics ===== */
            
            // do the actual HTTP POST request
            DataOutputStream wr = new DataOutputStream (
                  connection.getOutputStream());
            wr.writeBytes(body);
            wr.flush();
            
            
            // done, the validation service has got our file and has
            // returned a response with the results.
            // let us handle the response.
            BufferedReader reader = new BufferedReader( 
                    new InputStreamReader(connection.getInputStream()));
            
            while ((line = reader.readLine()) != null) {
                sbResults.append(line).append("\n");
            }
            
            wr.close();
            reader.close();
            
            // the results are back.
            results = sbResults.toString();
            
        } catch (ProtocolException ex) {
            Logger.getLogger(NuValidator.class.getName()).log(Level.SEVERE, null, ex);
        } catch (MalformedURLException ex) {
            Logger.getLogger(NuValidator.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(NuValidator.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            if(connection != null) {
                connection.disconnect(); 
            }
            return results;
        }
    }
}