package htmlvalidator;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.*;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class Validator implements XValidator 
{
    protected String validatorUrl;
    protected String validatorCharset;
    protected String validatorPathFile;
    protected String validatorPathUri;
    protected String validatorBodyFormat;
    
    @Override
    public int check (ArrayList documents) {
        int i;
        int n;
        String results;
        String doc;
        int exitCode = 0;
        
        for(i = 0, n = documents.size(); i < n; i++) {
            doc = documents.get(i).toString();
            
            if(Pattern.matches("https?://.*", doc)) {
                results = this.checkByUri(doc);
            } else {
                results = this.checkByFile(doc);                
            }
            
            // tell the audience what document we are at
            System.out.println(doc + ":");
                
            // TODO should propably be a bit more precise.
            Pattern p = Pattern.compile("error");
            Matcher m = p.matcher(results);
            
            if (m.find()) {
                exitCode = 1;
                System.out.println(results);
                break;
            }
        }
        
        return exitCode; // will halt ant if errors.
    }

    // sends a HTTP GET request.
    @Override
    public String checkByUri (String uri) {
        HttpURLConnection connection = null;
        String results = "";
        
        try {
            String line;
            StringBuilder sb = new StringBuilder();
            String safeUri = URLEncoder.encode(uri, this.validatorCharset);
            String completeUri = String.format(this.validatorPathUri, 
                    this.validatorUrl, safeUri);
            
            URL url = new URL(completeUri);

            connection = (HttpURLConnection) url.openConnection(); 

            connection.setRequestMethod("GET");
            connection.setReadTimeout(15 * 1000);
            
            connection.connect();

            BufferedReader reader = new BufferedReader( 
                    new InputStreamReader(connection.getInputStream()));
            
            while ((line = reader.readLine()) != null) {
                sb.append(line).append("\n");
            }
            
            reader.close();
            
            results = sb.toString();
            
        } catch (ProtocolException ex) {
            Logger.getLogger(NuValidator.class.getName()).log(Level.SEVERE, null, ex);
        } catch (MalformedURLException ex) {
            Logger.getLogger(NuValidator.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(NuValidator.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            if(connection != null) {
                connection.disconnect(); 
            }
            return results;
        }
    }

    @Override
    public String checkByFile(String filepath) {
        throw new UnsupportedOperationException("Not supported yet.");
    }
}
