package htmlvalidator;

import java.io.*;
import java.net.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class W3CValidator extends Validator implements XValidator 
{
    public W3CValidator (String url) {
        this.validatorUrl = url;
        this.validatorCharset = "UTF-8";
        
        this.validatorPathFile = "%s/check";
        this.validatorPathUri = "%s/check?uri=%s;output=json";
        this.validatorBodyFormat = "fragment=%s;output=json";
    }
    
    @Override
    public String checkByFile (String filepath) {
        HttpURLConnection connection = null;
        String results = ""; // response from the validation service
        
        try {
            String completeUri = String.format(this.validatorPathFile, 
                    this.validatorUrl);
            URL url = new URL(completeUri);
            connection = (HttpURLConnection) url.openConnection(); 
            StringBuilder sbResults = new StringBuilder();
            
            String line;
            StringBuilder sbFile = new StringBuilder();
            
            // prepare the connection 
            connection.setRequestMethod("POST");
            connection.setReadTimeout(15 * 1000);
            connection.setDoOutput(true);
            
            /* ===== w3c validator specifics ===== */
            
            // W3C validator wants the content of the file as a  
            // POST param, "fragment". The output format we want should be 
            // specified in the form ";output=x" at the end of the param
            // (we are using json).
            // We therefore read the file to memory and write it via an
            // output stream.
            
            File file = new File(filepath);
            FileInputStream fis = new FileInputStream(file);
            DataInputStream bis = new DataInputStream(fis);
            BufferedReader br = new BufferedReader(new InputStreamReader(bis, this.validatorCharset));

            while ((line = br.readLine()) != null) {
                sbFile.append(line).append("\n");
            }
            
            fis.close();
            bis.close();
            br.close();
            
            //ok, now we got the whole file in memory.
            String body = String.format(this.validatorBodyFormat, 
                    URLEncoder.encode(sbFile.toString(), this.validatorCharset));
            
            /* ===== /w3c validator specifics ===== */
            
            // do the actual HTTP POST request
            DataOutputStream wr = new DataOutputStream (
                  connection.getOutputStream());
            wr.writeBytes(body);
            wr.flush();
            
            // done, the validation service has got our file and has
            // returned a response with the results.
            // let us handle the response.
            BufferedReader reader = new BufferedReader( 
                    new InputStreamReader(connection.getInputStream()));
            
            while ((line = reader.readLine()) != null) {
                sbResults.append(line).append("\n");
            }
            
            wr.close();
            reader.close();
            
            // the results are back.
            results = sbResults.toString();
            
            /*
            String r = ".*{\\n[^\\n]*\"lastLine\": ([^\\n]+?),\\n[^\\n]*\"lastColumn\": ([^\\n]+?),\\n[^\n]*\"message\": \"([^\\n]+?)\".+?}";
            Pattern p = Pattern.compile(r, Pattern.MULTILINE);
            
            Matcher m = p.matcher(results);
            
            while(m.find()) {
                System.out.println(m.group());
            }
            */
            
        } catch (ProtocolException ex) {
            Logger.getLogger(NuValidator.class.getName()).log(Level.SEVERE, null, ex);
        } catch (MalformedURLException ex) {
            Logger.getLogger(NuValidator.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(NuValidator.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            if(connection != null) {
                connection.disconnect(); 
            }
            return results;
        }
    }
}